Index: src/_backend_agg.cpp
===================================================================
--- src/_backend_agg.cpp	(revision 7061)
+++ src/_backend_agg.cpp	(working copy)
@@ -104,6 +104,18 @@
     return Py::Object();
 }
 
+Py::Object BufferRegion::get_bounds(const Py::Tuple &args) {
+    args.verify_length(0);
+
+    Py::Tuple bounds(4);
+    bounds[0] = Py::Int(rect.x1);
+    bounds[1] = Py::Int(rect.y1);
+    bounds[2] = Py::Int(rect.x2);
+    bounds[3] = Py::Int(rect.y2);
+
+    return bounds;
+}
+
 Py::Object BufferRegion::to_string_argb(const Py::Tuple &args) {
   // owned=true to prevent memory leak
   Py_ssize_t length;
@@ -426,6 +438,54 @@
   return Py::Object();
 }
 
+Py::Object
+RendererAgg::restore_region2(const Py::Tuple& args) {
+  //copy BufferRegion to buffer
+  args.verify_length(7);
+
+
+
+  int x(0),y(0), xx1(0),yy1(0), xx2(0), yy2(0);
+  try {
+    xx1 = Py::Int( args[1] );
+    yy1 = Py::Int( args[2] );
+    xx2 = Py::Int( args[3] );
+    yy2 = Py::Int( args[4] );
+    x = Py::Int( args[5] );
+    y = Py::Int( args[6] );
+  }
+  catch (Py::TypeError) {
+    throw Py::TypeError("Invalid input arguments to draw_text_image");
+  }
+
+
+  BufferRegion* region  = static_cast<BufferRegion*>(args[0].ptr());
+
+  if (region->data==NULL)
+    throw Py::ValueError("Cannot restore_region from NULL data");
+    //return Py::Object();
+
+  //std::cout << "restoring " << region->width << " " << region->height << " " << region->stride << " " << region->rect.x1 << " " << region->rect.y1 << std::endl;
+
+  //agg::rect_i rect((int)l, height - (int)t, (int)r, height - (int)b);
+  //agg::rect_i rect(xx1-region->rect.x1, height - (yy2-region->rect.y1), 
+  //xx2-region->rect.x1, height - (yy1-region->rect.y1));
+  agg::rect_i rect(xx1-region->rect.x1, (yy1-region->rect.y1), 
+		   xx2-region->rect.x1, (yy2-region->rect.y1));
+
+
+  agg::rendering_buffer rbuf;
+  rbuf.attach(region->data,
+	      region->width,
+	      region->height,
+	      region->stride);
+
+  //rendererBase.copy_from(rbuf, 0, region->rect.x1, region->rect.y1);
+  rendererBase.copy_from(rbuf, &rect, x, y);
+
+  return Py::Object();
+}
+
 bool RendererAgg::render_clippath(const Py::Object& clippath, const agg::trans_affine& clippath_trans) {
   typedef agg::conv_transform<PathIterator> transformed_path_t;
   typedef agg::conv_curve<transformed_path_t> curve_t;
@@ -1717,6 +1777,9 @@
   add_varargs_method("set_y", &BufferRegion::set_y,
 		     "set_y(y)");
 
+  add_varargs_method("get_bounds", &BufferRegion::get_bounds,
+		     "get_bounds()");
+
   add_varargs_method("to_string", &BufferRegion::to_string,
 		     "to_string()");
   add_varargs_method("to_string_argb", &BufferRegion::to_string_argb,
@@ -1759,6 +1822,8 @@
  		     "copy_from_bbox(bbox)");
   add_varargs_method("restore_region", &RendererAgg::restore_region,
  		     "restore_region(region)");
+  add_varargs_method("restore_region2", &RendererAgg::restore_region2,
+ 		     "restore_region(region, x1, y1, x2, y2, x3, y3)");
 }
 
 extern "C"
Index: src/_backend_agg.h
===================================================================
--- src/_backend_agg.h	(revision 7061)
+++ src/_backend_agg.h	(working copy)
@@ -87,6 +87,8 @@
   Py::Object set_x(const Py::Tuple &args);
   Py::Object set_y(const Py::Tuple &args);
 
+  Py::Object get_bounds(const Py::Tuple &args);
+
   Py::Object to_string(const Py::Tuple &args);
   Py::Object to_string_argb(const Py::Tuple &args);
   static void init_type(void);
@@ -174,6 +176,7 @@
 
   Py::Object copy_from_bbox(const Py::Tuple & args);
   Py::Object restore_region(const Py::Tuple & args);
+  Py::Object restore_region2(const Py::Tuple & args);
 
   virtual ~RendererAgg();
 
